@@ -377,8 +377,12 @@ class Relation:
 
             relation.units[0].unit_name.split('/')[0]
         """
-        if self._application_name is None and self.units:
-            self._application_name = self.units[0].unit_name.split('/')[0]
+        if self._application_name is None:
+            if self.units:
+                self._application_name = self.units[0].unit_name.split('/')[0]
+            else:
+                # NOTE(ganso): This is a single unit peer relation corner case
+                return hookenv.application_name()
         return self._application_name
 
     @property
@@ -514,7 +518,7 @@ class Relation:
             # using JSONUnitDataView though it's name includes unit.
             self._remote_app_data = JSONUnitDataView(hookenv.relation_get(
                 app=self.application_name,
-                rid=self.relation.relation_id))
+                rid=self.relation_id))
         return self._remote_app_data
 
     @property
@@ -855,6 +859,8 @@ class UnitDataView(UserDict):
         return self._writeable
 
     def get(self, key, default=None):
+        if self.data is None:
+            return default
         return self.data.get(key, default)
 
     def __getitem__(self, key):
